from bolsonaro.models.omp_forest_classifier import OmpForestClassifier
from bolsonaro.models.omp_forest_regressor import OmpForestRegressor
from bolsonaro.data.task import Task
from bolsonaro.models.model_parameters import ModelParameters

import os
import pickle


class ModelFactory(object):

    @staticmethod
    def build(task, model_parameters):
        if task == Task.CLASSIFICATION:
            model_func = OmpForestClassifier
        elif task == Task.REGRESSION:
            model_func = OmpForestRegressor
        else:
            raise ValueError("Unsupported task '{}'".format(task))
        return model_func(model_parameters)

    @staticmethod
    def load(task, directory_path, experiment_id, model_raw_results):
        model_parameters = ModelParameters.load(directory_path, experiment_id)
        model = ModelFactory.build(task, model_parameters)
        model.set_forest(model_raw_results.forest)
        model.set_weights(model_raw_results.weights)
        return model
