from bolsonaro.models.model_raw_results import ModelRawResults
from bolsonaro.error_handling.logger_factory import LoggerFactory
from . import LOG_PATH

import time
import datetime
import numpy as np


class Trainer(object):

    def __init__(self, dataset):
        self._dataset = dataset
        self._logger = LoggerFactory.create(LOG_PATH, __name__)

    def train(self, model, models_dir):
        self._logger.debug('Training model using train set...')
        begin_time = time.time()

        if model.models_parameters.use_dev_subset:
            X_forest = self._dataset.X_train
            y_forest = self._dataset.y_train
            X_omp = self._dataset.X_dev
            y_omp = self._dataset.y_dev
            self._logger.debug('Fitting the forest on train subset and OMP on dev subset.')
        else:
            X_forest = np.concatenate([self._dataset.X_train, self._dataset.X_dev])
            X_omp = np.concatenate([self._dataset.X_train, self._dataset.X_dev])
            y_forest = np.concatenate([self._dataset.y_train, self._dataset.y_dev])
            y_omp = np.concatenate([self._dataset.y_train, self._dataset.y_dev])
            self._logger.debug('Fitting both the forest and OMP on train+dev subsets.')

        model.fit(
            X_forest=X_forest,
            y_forest=y_forest,
            X_omp=X_omp,
            y_omp=y_omp
        )
        end_time = time.time()

        ModelRawResults(
            forest=model.forest,
            weights=model.weights,
            training_time=end_time - begin_time,
            datetime=datetime.datetime.now(),
            train_score=model.score(self._dataset.X_train, self._dataset.y_train),
            dev_score=model.score(self._dataset.X_dev, self._dataset.y_dev),
            test_score=model.score(self._dataset.X_test, self._dataset.y_test),
            score_metric=model.DEFAULT_SCORE_METRIC, # TODO: resolve the used metric in a proper way
            train_score_regressor=model.score_regressor(self._dataset.X_train, self._dataset.y_train),
            dev_score_regressor=model.score_regressor(self._dataset.X_dev, self._dataset.y_dev),
            test_score_regressor=model.score_regressor(self._dataset.X_test, self._dataset.y_test)
        ).save(models_dir)
