import matplotlib.pyplot as plt
import numpy as np
from sklearn.neighbors.kde import KernelDensity


class Plotter(object):

    @staticmethod
    def weight_density(weights, X, file_path):
        X_plot = [np.exp(elem) for elem in weights]
        fig, ax = plt.subplots()

        for kernel in ['gaussian', 'tophat', 'epanechnikov']:
            kde = KernelDensity(kernel=kernel, bandwidth=0.5).fit(X_plot)
            log_dens = kde.score_samples(X_plot)
            ax.plot(X_plot, np.exp(log_dens), '-',
                    label="kernel = '{0}'".format(kernel))

        ax.legend(loc='upper left')
        ax.plot(X[:, 0], -0.005 - 0.01 * np.random.random(X.shape[0]), '+k')

        ax.set_xlim(-4, 9)
        ax.set_ylim(-0.02, 0.4)
        fig.savefig(file_path, dpi=fig.dpi)
        plt.close(fig)

    @staticmethod
    def plot_mean_and_CI(ax, mean, lb, ub, x_value, color_mean=None, facecolor=None, label=None):
        # plot the shaded range of the confidence intervals
        ax.fill_between(x_value, ub, lb, facecolor=facecolor, alpha=.5)
        # plot the mean on top
        ax.plot(x_value, mean, c=color_mean, label=label)

    @staticmethod
    def plot_losses(file_path, all_experiment_scores, x_value, xlabel, ylabel, all_labels, title):
        fig, ax = plt.subplots()

        n = len(all_experiment_scores)

        """
        Get as many different colors from the specified cmap (here nipy_spectral)
        as there are curve to plot.
        """
        colors = Plotter.get_colors_from_cmap(n)

        # For each curve to plot
        for i in range(n):
            # Retreive the scores in a list for each seed
            experiment_scores = list(all_experiment_scores[i].values())
            # Compute the mean and the std for the CI
            mean_experiment_scores = np.average(experiment_scores, axis=0)
            std_experiment_scores = np.std(experiment_scores, axis=0)
            # Plot the score curve with the CI
            Plotter.plot_mean_and_CI(
                ax=ax,
                mean=mean_experiment_scores,
                lb=mean_experiment_scores + std_experiment_scores,
                ub=mean_experiment_scores - std_experiment_scores,
                x_value=x_value,
                color_mean=colors[i],
                facecolor=colors[i],
                label=all_labels[i]
            )

        plt.xlabel(xlabel)
        plt.ylabel(ylabel)
        plt.title(title)
        plt.legend(loc='upper right')
        fig.savefig(file_path, dpi=fig.dpi)
        plt.close(fig)

    @staticmethod
    def get_colors_from_cmap(n_colors, colormap_name='nipy_spectral'):
        return [plt.get_cmap(colormap_name)(1. * i/n_colors) for i in range(n_colors)]
